package kr.pe.nowonbun.household2.browser;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import javax.websocket.CloseReason;
import javax.websocket.EncodeException;
import javax.websocket.EndpointConfig;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.ServerEndpoint;

import kr.pe.nowonbun.household2.common.HConfigurator;
import kr.pe.nowonbun.household2.common.HWSParams;
import kr.pe.nowonbun.household2.common.HWSParamsDecoder;
import kr.pe.nowonbun.household2.common.HWSParamsEncoder;
import kr.pe.nowonbun.household2.common.HWebsocket;
import kr.pe.nowonbun.household2.dao.CategoryDao;
import kr.pe.nowonbun.household2.dao.HouseholdDao;
import kr.pe.nowonbun.household2.entity.Category;
import kr.pe.nowonbun.household2.entity.Household;

@ServerEndpoint(	value="/ws/listSearch.ws",
					decoders = { HWSParamsDecoder.class },
					encoders = { HWSParamsEncoder.class },
					configurator = HConfigurator.class)
public class HBWList extends HWebsocket{
	@Override
	protected Class<?> setLogclass() {
		return HBWList.class;
	}
	@OnOpen
	public void handleOpen(Session session, EndpointConfig conf) {
		setSocketSession(session);
		setHttpSession(conf);
	}
	@OnMessage
	public void handleMessage(Session session, HWSParams input) 
			throws EncodeException,IOException,Exception{
		setSocketSession(session);
		SimpleDateFormat s = new SimpleDateFormat("yyyy-MM");
		if("init".equals(input.getFunc())){
			logger().info("list initialize!");
			String dateStr = s.format(new Date());
			logger().info("Date initialize - "+ dateStr);
			sendMessage(input.getFunc(), dateStr);
		}else if("search".equals(input.getFunc())){
			HouseholdDao householddao = new HouseholdDao();
			Calendar c = Calendar.getInstance();
			c.setTime(s.parse(input.getData().toString()));
			Calendar c1 = Calendar.getInstance();
			Calendar c2 = Calendar.getInstance();
			
			c1.clear();
			c2.clear();
			
			c1.set(Calendar.YEAR,c.get(Calendar.YEAR));
			c1.set(Calendar.MONTH,c.get(Calendar.MONTH));
			c1.set(Calendar.DATE, 1);
		
			c2.set(Calendar.YEAR,c.get(Calendar.YEAR));
			c2.set(Calendar.MONTH,c.get(Calendar.MONTH));
			c2.set(Calendar.DATE, 1);
			c2.add(Calendar.MONTH, 1);
			c2.add(Calendar.DATE, -1);
			
			logger().info("household Type - " + getUerInfoSession().getHouseholdtype());
			logger().info("startDate - " + c1.getTime());
			logger().info("endDate - " + c2.getTime());
			List<Household> list = null;
			try{
				list = householddao.findList(new Object[]{
										HouseholdDao.DEFAULTLIST,
										getUerInfoSession().getHouseholdtype(),
										c1.getTime(),
										c2.getTime()});
				logger().info("Search count - "+list.size());
			}catch(Exception e){
				logger().error("household list Searching error!");
				throw e;
			}
			try{
				sendMessage(input.getFunc(), list);
			}catch(EncodeException e){
				logger().error("send Error Exception - " + e);
				throw e;
			}catch(IOException e){
				logger().error("send Error Exception - " + e);
				throw e;
			}
		}else if("insert".equals(input.getFunc())){
			String message = input.getData().toString();
			logger().info("insert message - "+message);
			Household data = (Household)processMessage(message, Household.class);
			
			HouseholdDao dao = new HouseholdDao();
			Hashtable<String, Object> ret = new Hashtable<String, Object>();
			try{
				if(validate(data)){
					dao.TransactionBegin();
					Integer idx = data.getIdx();
					if(idx != null){
						Household entity = dao.findByPk(idx);
						if(!entity.getHouseholdtype().equals(getUerInfoSession().getHouseholdtype())){
							throw new Exception("householdType Bind Error!");
						}
						entity.setState("1");
						dao.update(entity);
						logger().info("data modify idx - "+data.getIdx());
						data.setIdx(null);
					}
					data.setState("0");
					if(data.getCategory() == null){
						data.setCategory(0l);
					}
					data.setHouseholdtype(getUerInfoSession().getHouseholdtype());
					data.setCreater(getUerInfoSession().getUserid());
					data.setCreatedate(new Date());
					dao.create(data);
					logger().info("data addication");
					dao.TransactionCommit();
					ret.put("result", "success");
					ret.put("idx", data.getIdx());
				}else{
					logger().error("Household data validate error!");
					logger().error("householdDate - "+data.getHouseholddate());
					logger().error("moneyType - "+data.getMoneytype());
					logger().error("contents - "+data.getContents());
					logger().error("money - "+data.getMoney());
					logger().error("other - "+data.getOther());
					logger().error("householdType - "+data.getHouseholdtype());
					logger().error("Category - "+data.getCategory());
					ret.put("result", "failed");
				}
			}catch(Exception e){
				dao.TransactionRollBack();
				logger().error("data update error - "+e);
				ret.put("result", "failed");
			}finally{
				sendMessage(input.getFunc(), ret);
			}
			
		}else if("delete".equals(input.getFunc())){
			String message = input.getData().toString();
			logger().info("delete message - "+message);
			Household data = (Household)processMessage(message, Household.class);
			
			HouseholdDao dao = new HouseholdDao();
			Hashtable<String, Object> ret = new Hashtable<String, Object>();
			
			try{
				dao.TransactionBegin();
				data = dao.findByPk(data.getIdx());
				if(!data.getHouseholdtype().equals(getUerInfoSession().getHouseholdtype())){
					throw new Exception("householdType Bind Error!");
				}
				data.setState("1");
				dao.update(data);
				logger().info("data addication");
				dao.TransactionCommit();
				ret.put("result", "success");
			}catch(Exception e){
				dao.TransactionRollBack();
				logger().error("data delete error - "+e);
				ret.put("result", "failed");
			}finally{
				sendMessage(input.getFunc(), ret);
			}
		}else if("categorySearch".equals(input.getFunc())){
			CategoryDao dao = new CategoryDao();
			
			try{
				List<Category> categorylist = dao.findList(new Object[]{	CategoryDao.DEFAULTLIST,
	 					getUerInfoSession().getHouseholdtype()});
				logger().info("CategorySearch count - "+categorylist.size());
				sendMessage(input.getFunc(), categorylist);
			}catch(Exception e){
				logger().error("category Search Error - " + e);
				throw e;
			}finally{
				
			}
		}else if("categoryUpdate".equals(input.getFunc())){
			String message = input.getData().toString();
			logger().info("Category Update message - "+message);
			
			Category data = (Category)processMessage(message, Category.class);
			
			CategoryDao dao = new CategoryDao();
			Hashtable<String, Object> ret = new Hashtable<String, Object>();
			try{
				dao.TransactionBegin();
				Integer idx = data.getIdx();
				if(idx != null){
					Category entity = dao.findByPk(idx);
					if(!entity.getHouseholdtype().equals(getUerInfoSession().getHouseholdtype())){
						throw new Exception("householdType Bind Error!");
					}
					entity.setState("1");
					dao.update(entity);
					logger().info("data modify idx - "+data.getIdx());
					data.setIdx(null);
				}
				data.setState("0");
				data.setCreatedate(new Date());
				data.setCreater(getUerInfoSession().getUserid());
				data.setHouseholdtype(getUerInfoSession().getHouseholdtype());
				dao.create(data);				
				dao.TransactionCommit();
				ret.put("result", "success");
			}catch(Exception e){
				dao.TransactionRollBack();
				logger().error("data update error - "+e);
				ret.put("result", "failed");
			}finally{
				sendMessage(input.getFunc(), ret);
			}
			
		}else if("categoryDelete".equals(input.getFunc())){
			String message = input.getData().toString();
			logger().info("Category Delete message - "+message);
			
			Category data = (Category)processMessage(message, Category.class);
			
			CategoryDao dao = new CategoryDao();
			Hashtable<String, Object> ret = new Hashtable<String, Object>();
			try{
				dao.TransactionBegin();
				data = dao.findByPk(data.getIdx());
				if(!data.getHouseholdtype().equals(getUerInfoSession().getHouseholdtype())){
					throw new Exception("householdType Bind Error!");
				}
				data.setState("1");
				dao.update(data);
				logger().info("data addication");
				dao.TransactionCommit();
				ret.put("result", "success");				
			}catch(Exception e){
				dao.TransactionRollBack();
				logger().error("data update error - "+e);
				ret.put("result", "failed");
			}finally{
				sendMessage(input.getFunc(), ret);
			}
		}
	}
	@OnClose
	public void handleClose(Session session, CloseReason reason) {
		setSocketSession(session);
		System.out.println("close");
	}
	@OnError
	public void handleError(Session session, Throwable error) {
		setSocketSession(session);
		System.out.println("error");
	}
	protected boolean validate(Household household){
		if(household.getHouseholddate() == null){
			return false;
		}else if(household.getMoneytype() == null || "".equals(household.getMoneytype())){
			return false;
		}else if(household.getContents() == null || "".equals(household.getContents())){
			return false;
		}else if(household.getMoney() == null){
			return false;
		}else if(household.getHouseholdtype() == null){
			//return false;
		}else if(household.getCategory() == null){
			return false;
		}
		return true;
	}
	
}
